define("ember-on-modifier/modifiers/on", ["exports", "ember-on-modifier/utils/event-listener"], function (_exports, _eventListener) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.__counts = __counts;
  _exports.default = void 0;

  /**
   * These are private API and only used for testing instrumentation.
   */
  let adds = 0;
  let removes = 0;

  function __counts() {
    return {
      adds,
      removes
    };
  }

  const assertValidEventOptions = true
  /* DEBUG */
  && (() => {
    const ALLOWED_EVENT_OPTIONS = ['capture', 'once', 'passive'];

    const joinOptions = options => options.map(o => "'".concat(o, "'")).join(', ');

    return function (eventOptions, eventName) {
      const invalidOptions = Object.keys(eventOptions).filter(o => !ALLOWED_EVENT_OPTIONS.includes(o));
      (true && !(invalidOptions.length === 0) && Ember.assert("ember-on-modifier: Provided invalid event options (".concat(joinOptions(invalidOptions), ") to '").concat(eventName, "' event listener. Only these options are valid: ").concat(joinOptions(ALLOWED_EVENT_OPTIONS)), invalidOptions.length === 0));
    };
  })();

  function setupListener(element, eventName, callback, eventOptions, params) {
    if (true
    /* DEBUG */
    ) assertValidEventOptions(eventOptions, eventName);
    (true && !(typeof eventName === 'string' && eventName.length > 1) && Ember.assert("ember-on-modifier: '".concat(eventName, "' is not a valid event name. It has to be a string with a minimum length of 1 character."), typeof eventName === 'string' && eventName.length > 1));
    (true && !(typeof callback === 'function') && Ember.assert("ember-on-modifier: '".concat(callback, "' is not a valid callback. Provide a function."), typeof callback === 'function'));
    (true && !(!Array.isArray(params) || params.length === 0) && Ember.deprecate("ember-on-modifier: Passing additional arguments to be partially applied to the event listener is deprecated in order to comply with the RFC. Use the '{{fn}}' helper instead: https://www.npmjs.com/package/ember-fn-helper-polyfill", !Array.isArray(params) || params.length === 0, {
      id: 'ember-on-modifier.partial-application',
      until: '1.0.0',
      url: 'https://github.com/emberjs/rfcs/blob/master/text/0471-on-modifier.md'
    }));

    if (Array.isArray(params) && params.length > 0) {
      const _callback = callback;

      callback = function (...args) {
        return _callback.call(this, ...params, ...args);
      };
    }

    adds++;
    (0, _eventListener.addEventListener)(element, eventName, callback, eventOptions);
    return callback;
  }

  function destroyListener(element, eventName, callback, eventOptions) {
    if (element && eventName && callback) {
      removes++;
      (0, _eventListener.removeEventListener)(element, eventName, callback, eventOptions);
    }
  }

  var _default = Ember._setModifierManager(() => ({
    createModifier() {
      return {
        element: null,
        eventName: undefined,
        callback: undefined,
        eventOptions: undefined
      };
    },

    installModifier(state, element, {
      positional: [eventName, callback, ...params],
      named: eventOptions
    }) {
      state.callback = setupListener(element, eventName, callback, eventOptions, params);
      state.element = element;
      state.eventName = eventName;
      state.params = params;
      state.eventOptions = eventOptions;
    },

    updateModifier(state, {
      positional: [eventName, callback, ...params],
      named: eventOptions
    }) {
      destroyListener(state.element, state.eventName, state.callback, state.eventOptions);
      state.callback = setupListener(state.element, eventName, callback, eventOptions, params);
      state.eventName = eventName;
      state.params = params;
      state.eventOptions = eventOptions;
    },

    destroyModifier({
      element,
      eventName,
      callback,
      eventOptions
    }) {
      destroyListener(element, eventName, callback, eventOptions);
    }

  }), class OnModifier {});

  _exports.default = _default;
});