define("ember-makeup/services/makeup", ["exports", "ember-makeup/config", "p-event"], function (_exports, _config, _pEvent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  class MakeupService extends Ember.Service.extend(Ember.Evented) {
    constructor(...args) {
      super(...args);
      this.classNamePrefix = _config.default.options.contextClassNamePrefix;
      this.themePaths = _config.default.themePaths;
      this.linkElement = void 0;
      this.computedStyle = void 0;
      this.isReady = false;
    }

    willDestroy() {
      super.willDestroy();
      this.destroyLinkElement();
    }

    createLinkElement(href) {
      this.destroyLinkElement();
      this.linkElement = document.createElement('link');
      this.linkElement.setAttribute('rel', 'stylesheet');
      this.linkElement.setAttribute('href', href);
      document.head.appendChild(this.linkElement);
      return (0, _pEvent.default)(this.linkElement, 'load');
    }

    destroyLinkElement() {
      if (this.linkElement && this.linkElement.parentElement) this.linkElement.parentElement.removeChild(this.linkElement);
      this.linkElement = undefined;
    }

    resolveContext(key) {
      if (!this.isReady) return undefined;
      const context = this.getPropertyValue(key);
      (true && !(Boolean(context)) && Ember.assert("Could not resolve context '".concat(key, "'."), Boolean(context)));
      return "".concat(this.classNamePrefix).concat(context);
    }

    getPropertyValue(key) {
      return this.computedStyle.getPropertyValue("--".concat(key)).trim();
    }

    async setTheme(themeName) {
      (true && !(themeName in this.themePaths) && Ember.assert("'".concat(themeName, "' is not in the list of known themes: ").concat(Object.keys(this.themePaths).join(', ')), themeName in this.themePaths));
      await this.createLinkElement(this.themePaths[themeName]);
      this.computedStyle = window.getComputedStyle(document.documentElement);
      this.isReady = true;
      this.trigger('theme-change');
    }

  }

  _exports.default = MakeupService;
});