"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const ember_cli_entities_1 = require("./lib/utils/ember-cli-entities");
const broccoli_debug_1 = __importDefault(require("broccoli-debug"));
const preprocessor_registry_1 = require("./plugins/preprocessor-registry");
const options_1 = require("./lib/options");
const ember_css_modules_1 = __importDefault(require("./plugins/ember-css-modules"));
const broccoli_merge_trees_1 = __importDefault(require("broccoli-merge-trees"));
const config_creator_1 = require("./plugins/broccoli/config-creator");
const themes_1 = require("./themes");
const addonPrototype = ember_cli_entities_1.addon({
    name: require(`${__dirname}/../package`).name,
    makeupOptions: undefined,
    themeProviders: undefined,
    parentAddon: undefined,
    /* static */ ThemeProvider: themes_1.ThemeProvider,
    get debugTree() {
        return broccoli_debug_1.default.buildDebugCallback(this.name);
    },
    included(includer) {
        this.computeOptions(includer);
        this.themeProviders = new themes_1.ThemeProviderRegistry(this.parent, includer, 'ember-makeup-theme-provider');
        this._super.included.call(this, includer);
    },
    computeOptions(includer) {
        if (this.makeupOptions)
            return;
        if (!includer.options && !this.app)
            throw new Error('Could not find parent options.');
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const parentOptions = includer.options || this.app.options;
        this.makeupOptions = options_1.computeOptions(parentOptions.makeup);
        if (this.parent.parent) {
            this.parentAddon = includer;
        }
    },
    shouldIncludeChildAddon(childAddon) {
        const disabledAddons = ['ember-css-modules', 'ember-cli-sass'];
        if (disabledAddons.includes(childAddon.name)) {
            return false;
        }
        return this._super.shouldIncludeChildAddon.call(this, childAddon);
    },
    /**
     * Integrate with other CSS processors, like ember-cli-sass.
     *
     * @see https://github.com/ember-cli/ember-cli-preprocess-registry#addon-usage
     */
    setupPreprocessorRegistry(type, registry) {
        preprocessor_registry_1.register(this, type, registry);
    },
    /**
     * Integrate with ember-css-modules.
     *
     * @see https://github.com/salsify/ember-css-modules/blob/master/docs/PLUGINS.md#plugins
     */
    createCssModulesPlugin(parent) {
        this.computeOptions(parent);
        return new ember_css_modules_1.default(parent, this);
    },
    filePathForTheme(themeName) {
        return `${this.name}/${themeName}.css`;
    },
    treeForAddon(tree) {
        const originalTree = this.debugTree(this._super.treeForAddon.call(this, tree), 'treeForAddon:input');
        // Only run for the root app.
        if (this.parentAddon)
            return originalTree;
        const configFile = config_creator_1.configCreatorJS(`${this.name}/config`, {
            options: this.makeupOptions,
            themePaths: Object.fromEntries(this.themeProviders
                .getThemeNames()
                .map(themeName => [
                themeName,
                path_1.join('/', this.filePathForTheme(themeName))
            ]))
        });
        const mergedTree = this.debugTree(new broccoli_merge_trees_1.default([originalTree, configFile], {
            annotation: 'ember-makeup:merge-config'
        }), 'treeForAddon:output');
        return mergedTree;
    },
    treeForPublic() {
        // Only run for the root app.
        if (this.parentAddon)
            return undefined;
        return this.debugTree(config_creator_1.configCreatorCSS({
            getFileName: themeName => this.filePathForTheme(themeName),
            contextClassNamePrefix: this.makeupOptions.contextClassNamePrefix,
            themes: this.themeProviders.getThemes()
        }), 'treeForStyles:output');
    }
});
exports.default = addonPrototype;
