"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const broccoli_file_creator_1 = require("broccoli-file-creator");
const postcss_1 = require("postcss");
const lodash_transform_1 = __importDefault(require("lodash.transform"));
const broccoli_merge_trees_1 = __importDefault(require("broccoli-merge-trees"));
const cssesc_1 = __importDefault(require("cssesc"));
const config_key_1 = require("../../../lib/config-key");
const isPrimitiveValue = (value) => typeof value === 'string' || typeof value === 'number';
const buildKey = (keyPrefix, key) => [...keyPrefix, key].join('.');
function throwKeyError(keyPrefix, key, message) {
    throw new TypeError(`'${buildKey(keyPrefix, key)}': ${message}`);
}
/* eslint-disable no-param-reassign */
const flattenTheme = (keyPrefix, acc = {
    contextless: {},
    contextual: {}
}) => (theme) => lodash_transform_1.default(theme, ({ contextless, contextual }, themeNode, key) => {
    if (typeof themeNode === 'object') {
        const nestedEntries = Object.entries(themeNode);
        if (nestedEntries.length === 0) {
            throwKeyError(keyPrefix, key, 'Empty nested config objects are not allowed.');
        }
        const contexts = nestedEntries.filter(([nestedKey]) => config_key_1.isContextKey(nestedKey));
        if (contexts.length !== 0 && contexts.length !== nestedEntries.length) {
            throwKeyError(keyPrefix, key, 'Mixing contextual keys and regular keys is not allowed.');
        }
        if (contexts.length > 0) {
            for (const [contextKey, value] of contexts) {
                if (!isPrimitiveValue(value)) {
                    throwKeyError(keyPrefix, key, 'Contextual configs must be primitive values.');
                }
                const contextName = config_key_1.contextFromContextKey(contextKey);
                if (!contextual[contextName]) {
                    contextual[contextName] = {};
                }
                contextual[contextName][buildKey(keyPrefix, key)] = value;
            }
            return;
        }
        flattenTheme([...keyPrefix, key], {
            contextless,
            contextual
        })(themeNode);
        return;
    }
    if (isPrimitiveValue(themeNode)) {
        contextless[buildKey(keyPrefix, key)] = themeNode;
        return;
    }
    throwKeyError(keyPrefix, key, `Unsupported value type '${themeNode}' (${typeof themeNode}).`);
}, acc);
/* eslint-enable no-param-reassign */
const flattenedThemeToRule = (selector, theme) => Object.entries(theme).reduce((builtRule, [key, value]) => builtRule.append(postcss_1.decl({
    prop: config_key_1.serializeConfigKey(key),
    value: String(value)
})), postcss_1.rule({ selector }));
function themeToCSS(theme, { contextClassNamePrefix }) {
    const { contextless, contextual } = flattenTheme([])(theme);
    const css = postcss_1.root();
    css.append(flattenedThemeToRule(':root', contextless));
    for (const [contextName, contextualTheme] of Object.entries(contextual)) {
        css.append(flattenedThemeToRule(`.${cssesc_1.default(`${contextClassNamePrefix}${contextName}`, {
            isIdentifier: true
        })}`, contextualTheme));
    }
    return css.toString();
}
function configCreatorCSS({ themes, contextClassNamePrefix, getFileName }) {
    return new broccoli_merge_trees_1.default(Object.entries(themes).map(([name, theme]) => new broccoli_file_creator_1.Class(getFileName(name), themeToCSS(theme, { contextClassNamePrefix }))), { annotation: 'ember-makeup:config-creator-css' });
}
exports.configCreatorCSS = configCreatorCSS;
