"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const postcss_1 = require("postcss");
const postcss_value_parser_1 = __importDefault(require("postcss-value-parser"));
const utils_1 = require("./utils");
const config_key_1 = require("../../lib/config-key");
exports.default = postcss_1.plugin('postcss-ember-makeup', ({ keyword = 'cfg', reportUsage, to } = {}) => {
    const needsTransformation = (value) => value.includes(`${keyword}(`);
    if (!to)
        throw new TypeError('Called without `to` file path option.');
    return root => {
        root.walkDecls(decl => {
            if (!needsTransformation(decl.value))
                return;
            const rule = decl.parent;
            if (!rule || rule.type !== 'rule') {
                throw decl.error(`'${keyword}' must only be used in declarations contained in rules.`, { word: keyword });
            }
            const originalValue = decl.value;
            decl.value = postcss_value_parser_1.default(decl.value)
                .walk(node => {
                if (node.type !== 'function' || node.value !== keyword)
                    return;
                const keyNode = utils_1.getKeyNodeFromFunctionNode(decl, node, keyword);
                keyNode.type = 'word';
                const key = keyNode.value;
                keyNode.value = config_key_1.serializeConfigKey(key);
                node.nodes = [keyNode];
                // turns `cfg` into `var`
                node.value = 'var';
                if (reportUsage) {
                    const fallback = node.nodes.length > 1
                        ? postcss_value_parser_1.default.stringify(node.nodes.slice(1))
                        : undefined;
                    reportUsage({
                        // @todo https://github.com/postcss/postcss/pull/1277
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        selectors: rule.selectors,
                        prop: decl.prop,
                        originalValue,
                        key,
                        fallback,
                        path: to
                    });
                }
            }, true)
                .toString();
        });
    };
});
