"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const postcss_1 = require("postcss");
const utils_1 = require("./utils");
/**
 * Transforms `@context` at-rules to `composes` declarations. These are then
 * picked up by ember-css-modules and exposed to the `local-class` helper, which
 * we override in order to live update.
 *
 * `@context` accepts an optional config key that defaults to `context`.
 *
 * ```css
 * .foo {
 *   @context;
 * }
 * .bar {
 *   @context other-context;
 * }
 * ```
 *
 * ```css
 * .foo {
 *   composes: 'ember-makeup/contexts/context' from global;
 * }
 * .bar {
 *   composes: 'ember-makeup/contexts/other-context' from global;
 * }
 * ```
 */
exports.default = postcss_1.plugin('postcss-ember-makeup:composes-context', options => {
    if (!options)
        throw new TypeError('Cannot be invoked without options.');
    const { contextKeyword, contextClassNamePrefix } = options;
    return root => {
        utils_1.assertNoDoubleColonAtRule(contextKeyword, root);
        root.walkAtRules(contextKeyword, atRule => {
            const configKey = atRule.params.length === 0 ? 'context' : atRule.params;
            utils_1.assertValidConfigKey(configKey, atRule);
            atRule.replaceWith(postcss_1.decl({
                prop: 'composes',
                value: `${contextClassNamePrefix}${configKey} from global`
            }));
        });
    };
});
