"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const postcss_value_parser_1 = require("postcss-value-parser");
const config_key_1 = require("../../lib/config-key");
function assertNoDoubleColonAtRule(keyword, container) {
    container.walkAtRules(`${keyword}:`, atRule => {
        throw atRule.error(`Use '@${keyword}' without a ':'.`, { word: ':' });
    });
}
exports.assertNoDoubleColonAtRule = assertNoDoubleColonAtRule;
function assertValidConfigKey(configKey, container) {
    if (!config_key_1.isValidConfigKey(configKey)) {
        if (config_key_1.containsQuotes(configKey)) {
            throw container.error(`Do not put the config key in quotes.`, {
                word: configKey
            });
        }
        throw container.error(`Invalid config key: '${configKey}'`, {
            word: configKey
        });
    }
}
exports.assertValidConfigKey = assertValidConfigKey;
function getKeyNodeFromFunctionNode(decl, node, keyword = 'cfg') {
    const [keyNode] = node.nodes;
    if (!keyNode) {
        const syntheticKeyNode = {
            type: 'string',
            value: decl.prop,
            quote: "'",
            unclosed: false
        };
        return syntheticKeyNode;
    }
    if ((keyNode.type !== 'word' && keyNode.type !== 'string') ||
        keyNode.value.length === 0) {
        throw decl.error(`The first parameter of '${keyword}' has to be a key, but you provided: ${postcss_value_parser_1.stringify(node)}`, { word: keyword });
    }
    return keyNode;
}
exports.getKeyNodeFromFunctionNode = getKeyNodeFromFunctionNode;
