"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = require(".");
const isThemeProvider = (keyword) => (addon) => addon.pkg && addon.pkg.keywords && addon.pkg.keywords.includes(keyword);
// @TODO: refactor to use broccoli, to support automatic rebuilds.
class ThemeProviderRegistry {
    constructor(parent, includer, keyword) {
        this.parent = parent;
        this.includer = includer;
        this.plugins = this.parent.addons
            .filter(isThemeProvider(keyword))
            .map(addon => this.instantiatePluginFor(addon))
            .concat(this.instantiatePluginFromIncluder(includer))
            .filter(Boolean);
        if (typeof includer.options
            .createEmberMakeupThemeProvider === 'function') {
            const plugin = this.instantiatePluginFor(includer.options);
            if (plugin)
                this.plugins.push(plugin);
        }
    }
    instantiatePluginFromIncluder(includer) {
        if (!includer.options ||
            typeof includer.options.createEmberMakeupThemeProvider !== 'function') {
            return undefined;
        }
        return includer.options.createEmberMakeupThemeProvider(this.parent, this.includer);
    }
    instantiatePluginFor(addon) {
        if (typeof addon.createEmberMakeupThemeProvider !==
            'function') {
            this.parent.ui.writeWarnLine(`Addon '${addon.name}' has no 'createEmberMakeupThemeProvider' hook.`);
            return;
        }
        const plugin = addon.createEmberMakeupThemeProvider(this.parent, this.includer);
        if (!(plugin instanceof _1.ThemeProvider)) {
            this.parent.ui.writeWarnLine(`Addon '${addon.name}' did not return a 'ThemeProvider' instance from its 'createEmberMakeupThemeProvider' hook.`);
        }
        return plugin;
    }
    getThemeNames() {
        return Object.keys(this.getThemes());
    }
    getThemes() {
        return Object.assign({}, ...this.plugins.map(plugin => plugin.getThemes()));
    }
}
exports.ThemeProviderRegistry = ThemeProviderRegistry;
